<?php declare(strict_types=1);

/**
 * This class uses Carbon for advanced date handling.
 * (c) Brian Nesbitt <brian@nesbot.com>
 *
 * @link http://carbon.nesbot.com/
 *       For the full copyright and license information, please view the LICENSE
 *       file that was distributed with this source code.
 */

namespace Infomaker\Everyware\Support;

use Carbon\Carbon;
use DateTimeZone;
use IntlDateFormatter;

/**
 * Class Date
 * @package Infomaker\Everyware\Support
 */
class Date extends Carbon
{

    /**
     * The format of a date in Open Content
     */
    public const OC = 'Y-m-d\TH:i:s\Z';

    /**
     * Contains an instance of the formatter used to translate dates into text
     *
     * @var IntlDateFormatter|null
     */
    protected static $formatter;

    /**
     * Contains the timezone set as an environment-variable on the server
     *
     * @var string
     */
    protected static $defaultTimezone;

    /**
     * Contains the local/language set as an environment-variable on the server
     *
     * @var string
     */
    protected static $defaultLocale;

    /**
     * Date constructor.
     *
     *
     * @param string|null              $time
     * @param DateTimeZone|string|null $tz
     */
    public function __construct($time = null, $tz = null)
    {
        static::$defaultTimezone = env('PHP_TIMEZONE');
        static::$defaultLocale = env('PHP_DEFAULT_LOCALE');

        if ( ! static::$defaultTimezone) {
            NewRelicLog::error('Missing Environment variable "PHP_TIMEZONE" There might be a problem using Dates on the page.');
        }
        if ( ! static::$defaultLocale) {
            NewRelicLog::error('Missing Environment variable "PHP_DEFAULT_LOCALE" There might be a problem using Dates on the page.');
        }

        static::setLocale(explode('_', Str::snake(static::$defaultLocale))[0]);
        date_default_timezone_set(static::$defaultTimezone);

        parent::__construct($time, $tz);
    }

    /**
     * Convert date into a format readable to Open Content
     * To avoid resetting timezone on the original date we create a copy first
     * @return string
     */
    public function toOcString(): string
    {
        $copy = $this->copy()->setTimezone('UTC'); // Reset Timezone

        return $copy->format(static::OC);
    }

    /**
     * Use the custom formatter to format and translate a date into the requested format
     *
     * @param $format
     *
     * @uses  IntlDateFormatter::formatObject()
     * @return string
     */
    public function toText($format): string
    {
        return static::formatter()->formatObject($this, $format);
    }

    /**
     * Create a date using the format from Open Content
     *
     * @param string $date
     *
     * @return static
     */
    public static function createFromOcString($date)
    {
        return static::createFromTimestamp(strtotime($date));
    }

    /**
     * Function for retrieving the formatter used to format dates into human readable string
     *
     * @return IntlDateFormatter
     */
    public static function formatter(): IntlDateFormatter
    {
        if (static::$formatter === null) {
            static::setFormatter(static::$defaultLocale);
        }

        return static::$formatter;
    }

    /**
     * Function for retrieving the formatter used to format dates into human readable string
     *
     * @param string $local
     *
     * @return void
     */
    public static function setFormatter($local = ''): void
    {
        static::$formatter = new IntlDateFormatter ($local, // string locale
            IntlDateFormatter::FULL, // int date type
            IntlDateFormatter::FULL // int time type
        );
    }
}
